@@ -1,4 +1,5 @@
-from __future__ import division
+from __future__ import division, absolute_import
+
 import numpy as np
 import pycuda.elementwise as elementwise
 from pytools import memoize, memoize_method
@@ -10,6 +11,8 @@ from pycuda.compyte.array import (
         ArrayFlags as _ArrayFlags,
         get_common_dtype as _get_common_dtype_base)
 from pycuda.characterize import has_double_support
+import six
+from six.moves import range, zip, reduce
 
 
 def _get_common_dtype(obj1, obj2):
@@ -18,7 +21,7 @@ def _get_common_dtype(obj1, obj2):
 
 # {{{ vector types
 
-class vec:
+class vec:  # noqa
     pass
 
 
@@ -98,7 +101,7 @@ def _splay_backend(n, dev):
         block_count = max_blocks
         threads_per_block = max_threads
 
-    #print "n:%d bc:%d tpb:%d" % (n, block_count, threads_per_block)
+    # print "n:%d bc:%d tpb:%d" % (n, block_count, threads_per_block)
     return (block_count, 1), (threads_per_block, 1, 1)
 
 
@@ -164,7 +167,7 @@ class GPUArray(object):
             for dim in shape:
                 s *= dim
         except TypeError:
-            assert isinstance(shape, (int, long, np.integer))
+            assert isinstance(shape, (int, int, np.integer))
             s = shape
             shape = (shape,)
 
@@ -187,7 +190,7 @@ class GPUArray(object):
 
             strides = tuple(strides)
 
-        self.shape = shape
+        self.shape = tuple(shape)
         self.dtype = dtype
         self.strides = strides
         self.mem_size = self.size = s
@@ -208,84 +211,65 @@ class GPUArray(object):
 
         self._grid, self._block = splay(self.mem_size)
 
+    @property
+    def ndim(self):
+        return len(self.shape)
+
     @property
     @memoize_method
     def flags(self):
         return _ArrayFlags(self)
 
-    def set(self, ary):
-        assert ary.size == self.size
-        assert ary.dtype == self.dtype
-        if ary.strides != self.strides:
+    def set(self, ary, async=False, stream=None):
+        if ary.size != self.size:
+            raise ValueError("ary and self must be the same size")
+        if ary.shape != self.shape:
             from warnings import warn
-            warn("Setting array from one with different strides/storage order. "
-                    "This will cease to work in 2013.x.",
+            warn("Setting array from one with different shape.",
                     stacklevel=2)
+            ary = ary.reshape(self.shape)
 
-        assert self.flags.forc
+        if ary.dtype != self.dtype:
+            raise ValueError("ary and self must have the same dtype")
 
         if self.size:
-            drv.memcpy_htod(self.gpudata, ary)
+            _memcpy_discontig(self, ary, async=async, stream=stream)
 
     def set_async(self, ary, stream=None):
-        assert ary.size == self.size
-        assert ary.dtype == self.dtype
-        if ary.strides != self.strides:
-            from warnings import warn
-            warn("Setting array from one with different strides/storage order. "
-                    "This will cease to work in 2013.x.",
-                    stacklevel=2)
+        return self.set(ary, async=True, stream=None)
 
-        assert self.flags.forc
-
-        if not ary.flags.forc:
-            raise RuntimeError("cannot asynchronously set from "
-                    "non-contiguous array")
-
-        if self.size:
-            drv.memcpy_htod_async(self.gpudata, ary, stream)
-
-    def get(self, ary=None, pagelocked=False):
+    def get(self, ary=None, pagelocked=False, async=False, stream=None):
         if ary is None:
             if pagelocked:
                 ary = drv.pagelocked_empty(self.shape, self.dtype)
             else:
                 ary = np.empty(self.shape, self.dtype)
 
-            ary = _as_strided(ary, strides=self.strides)
+            strides = _compact_strides(self)
+            ary = _as_strided(ary, strides=strides)
         else:
-            assert ary.size == self.size
-            assert ary.dtype == self.dtype
-            assert ary.flags.forc
+            if self.size != ary.size:
+                raise ValueError("self and ary must be the same size")
+            if self.shape != ary.shape:
+                from warnings import warn
+                warn("get() between arrays of different shape is deprecated "
+                        "and will be removed in PyCUDA 2017.x",
+                        DeprecationWarning, stacklevel=2)
+                ary = ary.reshape(self.shape)
 
-        assert self.flags.forc, "Array in get() must be contiguous"
+            if self.dtype != ary.dtype:
+                raise TypeError("self and ary must have the same dtype")
 
         if self.size:
-            drv.memcpy_dtoh(ary, self.gpudata)
+            _memcpy_discontig(ary, self, async=async, stream=stream)
         return ary
 
     def get_async(self, stream=None, ary=None):
-        if ary is None:
-            ary = drv.pagelocked_empty(self.shape, self.dtype)
-
-            ary = _as_strided(ary, strides=self.strides)
-        else:
-            assert ary.size == self.size
-            assert ary.dtype == self.dtype
-            assert ary.flags.forc
-
-        assert self.flags.forc, "Array in get() must be contiguous"
-
-        if self.size:
-            drv.memcpy_dtoh_async(ary, self.gpudata, stream)
-        return ary
+        return self.get(ary=ary, async=True, stream=stream)
 
     def copy(self):
-        if not self.flags.forc:
-            raise RuntimeError("only contiguous arrays may copied.")
-
         new = GPUArray(self.shape, self.dtype)
-        drv.memcpy_dtod(new.gpudata, self.gpudata, self.nbytes)
+        _memcpy_discontig(new, self)
         return new
 
     def __str__(self):
@@ -690,13 +674,28 @@ class GPUArray(object):
         return result
 
     def reshape(self, *shape):
+        """Gives a new shape to an array without changing its data."""
+
         # TODO: add more error-checking, perhaps
+        if not self.flags.forc:
+            raise RuntimeError("only contiguous arrays may "
+                    "be used as arguments to this operation")
+
         if isinstance(shape[0], tuple) or isinstance(shape[0], list):
             shape = tuple(shape[0])
 
         if shape == self.shape:
             return self
 
+        if -1 in shape:
+            shape = list(shape)
+            idx = shape.index(-1)
+            size = -reduce(lambda x, y: x * y, shape, 1)
+            shape[idx] = self.size // size
+            if -1 in shape[idx:]:
+                raise ValueError("can only specify one unknown dimension")
+            shape = tuple(shape)
+
         size = reduce(lambda x, y: x * y, shape, 1)
         if size != self.size:
             raise ValueError("total size of new array must be unchanged")
@@ -743,6 +742,38 @@ class GPUArray(object):
                 base=self,
                 gpudata=int(self.gpudata))
 
+    def transpose(self, axes=None):
+        """Permute the dimensions of an array.
+
+        :arg axes: list of ints, optional.
+            By default, reverse the dimensions, otherwise permute the axes
+            according to the values given.
+
+        :returns: :class:`GPUArray` A view of the array with its axes permuted.
+
+        .. versionadded:: 2015.2
+        """
+
+        if axes is None:
+            axes = range(self.ndim-1, -1, -1)
+        if len(axes) != len(self.shape):
+            raise ValueError("axes don't match array")
+        new_shape = [self.shape[axes[i]] for i in range(len(axes))]
+        new_strides = [self.strides[axes[i]] for i in range(len(axes))]
+        return GPUArray(shape=tuple(new_shape),
+                        dtype=self.dtype,
+                        allocator=self.allocator,
+                        base=self.base or self,
+                        gpudata=self.gpudata,
+                        strides=tuple(new_strides))
+
+    @property
+    def T(self):  # noqa
+        """
+        .. versionadded:: 2015.2
+        """
+        return self.transpose()
+
     # {{{ slicing
 
     def __getitem__(self, index):
@@ -772,7 +803,7 @@ class GPUArray(object):
 
                 array_stride = self.strides[array_axis]
 
-                new_shape.append((stop-start)//idx_stride)
+                new_shape.append((stop-start-1)//idx_stride+1)
                 new_strides.append(idx_stride*array_stride)
                 new_offset += array_stride*start
 
@@ -810,6 +841,11 @@ class GPUArray(object):
                             "more than one ellipsis not allowed in index")
                 seen_ellipsis = True
 
+            elif index_entry is np.newaxis:
+                new_shape.append(1)
+                new_strides.append(0)
+                index_axis += 1
+
             else:
                 raise IndexError("invalid subindex in axis %d" % index_axis)
 
@@ -827,6 +863,9 @@ class GPUArray(object):
                 gpudata=int(self.gpudata)+new_offset,
                 strides=tuple(new_strides))
 
+    def __setitem__(self, index, value):
+        _memcpy_discontig(self[index], value)
+
     # }}}
 
     # {{{ complex-valued business
@@ -909,14 +948,14 @@ class GPUArray(object):
 
 def to_gpu(ary, allocator=drv.mem_alloc):
     """converts a numpy array to a GPUArray"""
-    result = GPUArray(ary.shape, ary.dtype, allocator, strides=ary.strides)
+    result = GPUArray(ary.shape, ary.dtype, allocator, strides=_compact_strides(ary))
     result.set(ary)
     return result
 
 
 def to_gpu_async(ary, allocator=drv.mem_alloc, stream=None):
     """converts a numpy array to a GPUArray"""
-    result = GPUArray(ary.shape, ary.dtype, allocator, strides=ary.strides)
+    result = GPUArray(ary.shape, ary.dtype, allocator, strides=_compact_strides(ary))
     result.set_async(ary, stream)
     return result
 
@@ -993,7 +1032,7 @@ def arange(*args, **kwargs):
         raise ValueError("too many arguments")
 
     admissible_names = ["start", "stop", "step", "dtype"]
-    for k, v in kwargs.iteritems():
+    for k, v in six.iteritems(kwargs):
         if k in admissible_names:
             if getattr(inf, k) is None:
                 setattr(inf, k, v)
@@ -1035,10 +1074,147 @@ def arange(*args, **kwargs):
 
 # }}}
 
+def _compact_strides(a):
+    # Compute strides to have same order as self, but packed
+    info = sorted((a.strides[axis], a.shape[axis], axis) for axis in xrange(len(a.shape)))
+
+    strides = [None]*len(a.shape)
+    stride = a.dtype.itemsize
+    for _, dim, axis in info:
+        strides[axis] = stride
+        stride *= dim
+    return strides
+
+def _memcpy_discontig(dst, src, async=False, stream=None):
+    """Copy the contents of src into dst.
+
+    The two arrays should have the same dtype, shape, and order, but
+    not necessarily the same strides. There may be up to _two_
+    axes along which either `src` or `dst` is not contiguous.
+    """
+
+    if not isinstance(src, (GPUArray, np.ndarray)):
+        raise TypeError("src must be GPUArray or ndarray")
+    if not isinstance(dst, (GPUArray, np.ndarray)):
+        raise TypeError("dst must be GPUArray or ndarray")
+    if src.shape != dst.shape:
+        raise ValueError("src and dst must be same shape")
+    if src.dtype != dst.dtype:
+        raise TypeError("src and dst must have same dtype")
+
+    # ndarray -> ndarray
+    if isinstance(src, np.ndarray) and isinstance(dst, np.ndarray):
+        dst[...] = src
+        return
+
+    if src.flags.forc and dst.flags.forc:
+        shape = [src.size]
+        src_strides = dst_strides = [src.dtype.itemsize]
+    else:
+        # put src in Fortran order (which should put dst in Fortran order too)
+        # and remove singleton axes
+        src_info = sorted((src.strides[axis], axis) for axis in xrange(len(src.shape)) if src.shape[axis] > 1)
+        axes = [axis for _, axis in src_info]
+        shape = [src.shape[axis] for axis in axes]
+        src_strides = [src.strides[axis] for axis in axes]
+        dst_strides = [dst.strides[axis] for axis in axes]
+
+        # copy functions require contiguity in minor axis, so add new axis if needed
+        if len(shape) == 0 or src_strides[0] != src.dtype.itemsize or dst_strides[0] != dst.dtype.itemsize:
+            shape[0:0] = [1]
+            src_strides[0:0] = [0]
+            dst_strides[0:0] = [0]
+            axes[0:0] = [np.newaxis]
+
+        # collapse contiguous dimensions
+        # and check that dst is in same order as src
+        i = 1
+        while i < len(shape):
+            if dst_strides[i] < dst_strides[i-1]:
+                raise ValueError("src and dst must have same order")
+            if (src_strides[i-1] * shape[i-1] == src_strides[i] and
+                dst_strides[i-1] * shape[i-1] == dst_strides[i]):
+                shape[i-1:i+1] = [shape[i-1] * shape[i]]
+                del src_strides[i]
+                del dst_strides[i]
+                del axes[i]
+            else:
+                i += 1
+
+    if len(shape) <= 1:
+        if isinstance(src, GPUArray):
+            if isinstance(dst, GPUArray):
+                if async:
+                    drv.memcpy_dtod_async(dst.gpudata, src.gpudata, src.nbytes, stream=stream)
+                else:
+                    drv.memcpy_dtod(dst.gpudata, src.gpudata, src.nbytes)
+            else:
+                # The arrays might be contiguous in the sense of
+                # having no gaps, but the axes could be transposed
+                # so that the order is neither Fortran or C.
+                # So, we attempt to get a contiguous view of dst.
+                dst = _as_strided(dst, shape=(dst.size,), strides=(dst.dtype.itemsize,))
+                if async:
+                    drv.memcpy_dtoh_async(dst, src.gpudata, stream=stream)
+                else:
+                    drv.memcpy_dtoh(dst, src.gpudata)
+        else:
+            src = _as_strided(src, shape=(src.size,), strides=(src.dtype.itemsize,))
+            if async:
+                drv.memcpy_htod_async(dst.gpudata, src, stream=stream)
+            else:
+                drv.memcpy_htod(dst.gpudata, src)
+        return
+
+    if len(shape) == 2:
+        copy = drv.Memcpy2D()
+    elif len(shape) == 3:
+        copy = drv.Memcpy3D()
+    else:
+        raise ValueError("more than 2 discontiguous axes not supported %s" % (tuple(sorted(axes)),))
+
+    if isinstance(src, GPUArray):
+        copy.set_src_device(src.gpudata)
+    else:
+        copy.set_src_host(src)
+
+    if isinstance(dst, GPUArray):
+        copy.set_dst_device(dst.gpudata)
+    else:
+        copy.set_dst_host(dst)
+
+    copy.width_in_bytes = src.dtype.itemsize*shape[0]
+
+    copy.src_pitch = src_strides[1]
+    copy.dst_pitch = dst_strides[1]
+    copy.height = shape[1]
+
+    if len(shape) == 2:
+        if async:
+            copy(stream=stream)
+        else:
+            copy(aligned=True)
+
+    else: # len(shape) == 3
+        if src_strides[2] % src_strides[1] != 0:
+            raise RuntimeError("src's major stride must be a multiple of middle stride")
+        copy.src_height = src_strides[2] // src_strides[1]
+
+        if dst_strides[2] % dst_strides[1] != 0:
+            raise RuntimeError("dst's major stride must be a multiple of middle stride")
+        copy.dst_height = dst_strides[2] // dst_strides[1]
+
+        copy.depth = shape[2]
+        if async:
+            copy(stream=stream)
+        else:
+            copy()
+
+
 # {{{ pickle support
 
-import copy_reg
-copy_reg.pickle(GPUArray,
+import six.moves.copyreg
+six.moves.copyreg.pickle(GPUArray,
                 lambda data: (to_gpu, (data.get(),)),
                 to_gpu)
 
@@ -1219,6 +1395,34 @@ def multi_put(arrays, dest_indices, dest_shape=None, out=None, stream=None):
 # }}}
 
 
+# {{{ shape manipulation
+
+def transpose(a, axes=None):
+    """Permute the dimensions of an array.
+
+    :arg a: :class:`GPUArray`
+    :arg axes: list of ints, optional.
+        By default, reverse the dimensions, otherwise permute the axes
+        according to the values given.
+
+    :returns: :class:`GPUArray` A view of the array with its axes permuted.
+
+    .. versionadded:: 2015.2
+    """
+    return a.transpose(axes)
+
+
+def reshape(a, shape):
+    """Gives a new shape to an array without changing its data.
+
+    .. versionadded:: 2015.2
+    """
+
+    return a.reshape(shape)
+
+# }}}
+
+
 # {{{ conditionals
 
 def if_positive(criterion, then_, else_, out=None, stream=None):
@@ -1278,30 +1482,30 @@ maximum = _make_binary_minmax_func("max")
 
 # {{{ reductions
 
-def sum(a, dtype=None, stream=None):
+def sum(a, dtype=None, stream=None, allocator=None):
     from pycuda.reduction import get_sum_kernel
     krnl = get_sum_kernel(dtype, a.dtype)
-    return krnl(a, stream=stream)
+    return krnl(a, stream=stream, allocator=allocator)
 
 
-def subset_sum(subset, a, dtype=None, stream=None):
+def subset_sum(subset, a, dtype=None, stream=None, allocator=None):
     from pycuda.reduction import get_subset_sum_kernel
     krnl = get_subset_sum_kernel(dtype, subset.dtype, a.dtype)
     return krnl(subset, a, stream=stream)
 
 
-def dot(a, b, dtype=None, stream=None):
+def dot(a, b, dtype=None, stream=None, allocator=None):
     from pycuda.reduction import get_dot_kernel
     if dtype is None:
         dtype = _get_common_dtype(a, b)
     krnl = get_dot_kernel(dtype, a.dtype, b.dtype)
-    return krnl(a, b, stream=stream)
+    return krnl(a, b, stream=stream, allocator=allocator)
 
 
-def subset_dot(subset, a, b, dtype=None, stream=None):
+def subset_dot(subset, a, b, dtype=None, stream=None, allocator=None):
     from pycuda.reduction import get_subset_dot_kernel
     krnl = get_subset_dot_kernel(dtype, subset.dtype, a.dtype, b.dtype)
-    return krnl(subset, a, b, stream=stream)
+    return krnl(subset, a, b, stream=stream, allocator=allocator)
 
 
 def _make_minmax_kernel(what):
