@@ -240,6 +240,11 @@ namespace pycuda
 
       static const char *curesult_to_str(CUresult e)
       {
+#if CUDAPP_CUDA_VERSION >= 6000
+        const char* errstr;
+        cuGetErrorString(e, &errstr);
+        return errstr;
+#else
         switch (e)
         {
           case CUDA_SUCCESS: return "success";
@@ -337,6 +342,7 @@ namespace pycuda
 
           default: return "invalid/unknown error code";
         }
+#endif
       }
   };
 
@@ -1624,7 +1630,6 @@ namespace pycuda
           {
             scoped_context_activation ca(get_context());
             CUDAPP_CALL_GUARDED_CLEANUP(cuIpcCloseMemHandle, (m_devptr));
-            mem_free(m_devptr);
           }
           CUDAPP_CATCH_CLEANUP_ON_DEAD_CONTEXT(ipc_mem_handle);
 
@@ -1666,7 +1671,7 @@ namespace pycuda
     { \
       srcMemoryType = CU_MEMORYTYPE_HOST; \
       py_buffer_wrapper buf_wrapper; \
-      buf_wrapper.get(buf_py.ptr(), PyBUF_ANY_CONTIGUOUS); \
+      buf_wrapper.get(buf_py.ptr(), PyBUF_STRIDED_RO); \
       srcHost = buf_wrapper.m_buf.buf; \
     } \
     \
@@ -1686,7 +1691,7 @@ namespace pycuda
     { \
       dstMemoryType = CU_MEMORYTYPE_HOST; \
       py_buffer_wrapper buf_wrapper; \
-      buf_wrapper.get(buf_py.ptr(), PyBUF_ANY_CONTIGUOUS | PyBUF_WRITABLE); \
+      buf_wrapper.get(buf_py.ptr(), PyBUF_STRIDED); \
       dstHost = buf_wrapper.m_buf.buf; \
     } \
     \
