@@ -1,9 +1,12 @@
 #! /usr/bin/env python
+
+from __future__ import absolute_import, print_function
 import numpy as np
 import numpy.linalg as la
 import sys
 from pycuda.tools import mark_cuda_test
 from pycuda.characterize import has_double_support
+from six.moves import range
 
 
 def have_pycuda():
@@ -613,7 +616,7 @@ class TestGPUArray:
         min_a_b_gpu = gpuarray.minimum(a_gpu, b_gpu)
 
         print (max_a_b_gpu)
-        print (np.maximum(a, b))
+        print((np.maximum(a, b)))
 
         assert la.norm(max_a_b_gpu.get() - np.maximum(a, b)) == 0
         assert la.norm(min_a_b_gpu.get() - np.minimum(a, b)) == 0
@@ -732,9 +735,9 @@ class TestGPUArray:
     def test_stride_preservation(self):
         A = np.random.rand(3, 3)
         AT = A.T
-        print (AT.flags.f_contiguous, AT.flags.c_contiguous)
+        print((AT.flags.f_contiguous, AT.flags.c_contiguous))
         AT_GPU = gpuarray.to_gpu(AT)
-        print (AT_GPU.flags.f_contiguous, AT_GPU.flags.c_contiguous)
+        print((AT_GPU.flags.f_contiguous, AT_GPU.flags.c_contiguous))
         assert np.allclose(AT_GPU.get(), AT)
 
     @mark_cuda_test
@@ -758,6 +761,18 @@ class TestGPUArray:
         a_gpu.reshape((4, 32))
         a_gpu.reshape([4, 32])
 
+        # using -1 as unknown dimension
+        assert a_gpu.reshape(-1, 32).shape == (4, 32)
+        assert a_gpu.reshape((32, -1)).shape == (32, 4)
+        assert a_gpu.reshape(((8, -1, 4))).shape == (8, 4, 4)
+
+        throws_exception = False
+        try:
+            a_gpu.reshape(-1, -1, 4)
+        except ValueError:
+            throws_exception = True
+        assert throws_exception
+
     @mark_cuda_test
     def test_view(self):
         a = np.arange(128).reshape(8, 16).astype(np.float32)
@@ -840,6 +855,52 @@ class TestGPUArray:
         assert minmax["cur_min"] == np.min(a)
         assert minmax["cur_max"] == np.max(a)
 
+    @mark_cuda_test
+    def test_sum_allocator(self):
+        import pycuda.tools
+        pool = pycuda.tools.DeviceMemoryPool()
+
+        rng = np.random.randint(low=512,high=1024)
+
+        a = gpuarray.arange(rng,dtype=np.int32)
+        b = gpuarray.sum(a)
+        c = gpuarray.sum(a, allocator=pool.allocate)
+
+        # Test that we get the correct results
+        assert b.get() == rng*(rng-1)//2
+        assert c.get() == rng*(rng-1)//2
+
+        # Test that result arrays were allocated with the appropriate allocator
+        assert b.allocator == a.allocator
+        assert c.allocator == pool.allocate
+
+    @mark_cuda_test
+    def test_dot_allocator(self):
+        import pycuda.tools
+        pool = pycuda.tools.DeviceMemoryPool()
+
+        a_cpu = np.random.randint(low=512,high=1024,size=1024)
+        b_cpu = np.random.randint(low=512,high=1024,size=1024)
+
+        # Compute the result on the CPU
+        dot_cpu_1 = np.dot(a_cpu, b_cpu)
+
+        a_gpu = gpuarray.to_gpu(a_cpu)
+        b_gpu = gpuarray.to_gpu(b_cpu)
+
+        # Compute the result on the GPU using different allocators
+        dot_gpu_1 = gpuarray.dot(a_gpu, b_gpu)
+        dot_gpu_2 = gpuarray.dot(a_gpu, b_gpu, allocator=pool.allocate)
+
+        # Test that we get the correct results
+        assert dot_cpu_1 == dot_gpu_1.get()
+        assert dot_cpu_1 == dot_gpu_2.get()
+
+        # Test that result arrays were allocated with the appropriate allocator
+        assert dot_gpu_1.allocator == a_gpu.allocator
+        assert dot_gpu_2.allocator == pool.allocate
+
+
     @mark_cuda_test
     def test_view_and_strides(self):
         from pycuda.curandom import rand as curand
@@ -851,9 +912,7 @@ class TestGPUArray:
         assert y.shape == Y.shape
         assert y.strides == Y.strides
 
-        import pytest
-        with pytest.raises(AssertionError):
-            assert (y.get() == X.get()[:3, :5]).all()
+        assert np.array_equal(y.get(), X.get()[:3, :5])
 
     @mark_cuda_test
     def test_scalar_comparisons(self):
@@ -892,6 +951,71 @@ class TestGPUArray:
         assert la.norm(max_a0_gpu.get() - np.maximum(a, 0)) == 0
         assert la.norm(min_a0_gpu.get() - np.minimum(0, a)) == 0
 
+    @mark_cuda_test
+    def test_transpose(self):
+        import pycuda.gpuarray as gpuarray
+        from pycuda.curandom import rand as curand
+
+        a_gpu = curand((10,20,30))
+        a = a_gpu.get()
+
+        #assert np.allclose(a_gpu.transpose((1,2,0)).get(), a.transpose((1,2,0))) # not contiguous
+        assert np.allclose(a_gpu.T.get(), a.T)
+
+    @mark_cuda_test
+    def test_newaxis(self):
+        import pycuda.gpuarray as gpuarray
+        from pycuda.curandom import rand as curand
+
+        a_gpu = curand((10,20,30))
+        a = a_gpu.get()
+
+        b_gpu = a_gpu[:,np.newaxis]
+        b = a[:,np.newaxis]
+
+        assert b_gpu.shape == b.shape
+        assert b_gpu.strides == b.strides
+
+    @mark_cuda_test
+    def test_copy(self):
+        from pycuda.curandom import rand as curand
+        a_gpu = curand((3,3))
+
+        for start, stop, step in [(0,3,1), (1,2,1), (0,3,2), (0,3,3)]:
+            assert np.allclose(a_gpu[start:stop:step].get(), a_gpu.get()[start:stop:step])
+
+        a_gpu = curand((3,1))
+        for start, stop, step in [(0,3,1), (1,2,1), (0,3,2), (0,3,3)]:
+            assert np.allclose(a_gpu[start:stop:step].get(), a_gpu.get()[start:stop:step])
+
+        a_gpu = curand((3,3,3))
+        for start, stop, step in [(0,3,1), (1,2,1), (0,3,2), (0,3,3)]:
+            assert np.allclose(a_gpu[start:stop:step,start:stop:step].get(), a_gpu.get()[start:stop:step,start:stop:step])
+
+        a_gpu = curand((3,3,3)).transpose((1,2,0))
+        a = a_gpu.get()
+        for start, stop, step in [(0,3,1), (1,2,1), (0,3,2), (0,3,3)]:
+            assert np.allclose(a_gpu[start:stop:step,:,start:stop:step].get(), a_gpu.get()[start:stop:step,:,start:stop:step])
+
+        # 4-d should work as long as only 2 axes are discontiguous
+        a_gpu = curand((3,3,3,3))
+        a = a_gpu.get()
+        for start, stop, step in [(0,3,1), (1,2,1), (0,3,3)]:
+            assert np.allclose(a_gpu[start:stop:step,:,start:stop:step].get(), a_gpu.get()[start:stop:step,:,start:stop:step])
+
+    @mark_cuda_test
+    def test_get_set(self):
+        import pycuda.gpuarray as gpuarray
+
+        a = np.random.normal(0., 1., (4,4))
+        a_gpu = gpuarray.to_gpu(a)
+        assert np.allclose(a_gpu.get(), a)
+        assert np.allclose(a_gpu[1:3,1:3].get(), a[1:3,1:3])
+
+        a = np.random.normal(0., 1., (4,4,4)).transpose((1,2,0))
+        a_gpu = gpuarray.to_gpu(a)
+        assert np.allclose(a_gpu.get(), a)
+        assert np.allclose(a_gpu[1:3,1:3,1:3].get(), a[1:3,1:3,1:3])
 
 if __name__ == "__main__":
     # make sure that import failures get reported, instead of skipping the tests.
